"""
Utility functions for code output formatting
"""

import base64
import io
import os
import tempfile

# TODO making altair and bokeh charts non-interactive (png)
# for pdfs possible, but would require js dependencies see
# https://pypi.org/project/altair-saver/
# https://docs.bokeh.org/en/latest/docs/user_guide/export.html#additional-dependencies

def output_altair(figure):
    """
    Process altair figure
    """
    temp = tempfile.NamedTemporaryFile(delete=False, suffix=".html")
    figure.save(temp.name)
    with open(temp.name, "r") as f:
        rendered_figure = f.read()
    os.remove(temp.name)
    return rendered_figure

def output_bokeh(figure):
    """
    Process boheh figure
    """
    from bokeh.embed import file_html
    from bokeh.resources import CDN
    return file_html(figure, CDN, "Bokeh plot")

def output_matplotlib(figure):
    """
    Process matplotlib figure
    """
    fig_bytes = io.BytesIO()
    figure.savefig(fig_bytes, format="png")
    fig_bytes.seek(0)
    return _bytes_to_html(fig_bytes.read())

def output_plotly(figure, interactive=True):
    """
    Process plotly figure
    """
    import plotly
    if interactive:
        return plotly.io.to_html(figure, include_plotlyjs="cdn")
    else:
        img_bytes = figure.to_image(format="png")
        return _bytes_to_html(img_bytes)

def _bytes_to_html(bytes):
    """
    Helper for getting base64 encoded img html tag
    """
    img_encoded = base64.b64encode(bytes).decode()
    img_html = f"""<img src="data:image/png;base64,{img_encoded}" />"""
    return img_html




"""merkury

Usage:
    merkury [-o <file>] [-f <format>] [-t <theme>] <script>

Options:
    -h --help                       Show this screen.
    -o <file>, --output <file>      Specify output file (if missing, "<script_name>_<date>").
    -f <format>, --format <format>  Specify format: html (default), pdf.
    -t <theme>, --theme <theme>     Specify color theme: dark (default), light. Valid for HTML output.
    -v, --version                   Show version.
"""

from .renderer import get_default_path, produce_report
from .runner import execute
from docopt import docopt
from pathlib import Path

FORMATS = ("html", "pdf", )
THEMES = ("dark", "light", )

def main():
    """
    Program entrypoint
    """
    args = docopt(__doc__, version="merkury 0.3")
    format = (args.get("--format") or "html").lower()
    assert format in FORMATS, f"Unknown format: {format}. Options: html, pdf"
    color_theme = (args.get("--theme") or "dark").lower()
    assert color_theme in THEMES, f"Unknown color theme: {color_theme}. Options: dark, light"
    python_file_path = Path(args.get("<script>"))
    output_file_path = Path(args.get("--output") or get_default_path(python_file_path, format))
    code = execute(python_file_path)
    produce_report(code, format, color_theme, python_file_path, output_file_path)

if __name__ == "__main__":
    main()



"""
Reformats code output into report.
"""

from datetime import datetime
from importlib.metadata import version
from jinja2 import Environment, PackageLoader
from markdown import markdown
from pathlib import Path
import pdfkit
import re

jinja = Environment(
    loader=PackageLoader("merkury", "templates")
)
jinja.filters["markdown"] = lambda content: markdown(content, extensions=["tables", ])

def join_chunks(code):
    """
    Join code nodes without anything printed
    """
    in_chunk = out_chunk = ""
    html = markdown = False
    for input, output in code:
        html = html or any((bool(re.match("^#HTML", line)) for line in input))
        markdown = markdown or any((bool(re.match("^#MARKDOWN", line)) for line in input))
        in_chunk += "".join((line+"\n" for line in input))
        if output != "":
            out_chunk += output
            assert (sum([html, markdown]) <= 1), "Only one option can be specified"
            yield {"in": in_chunk, "out": out_chunk, "html": html, "markdown": markdown}
            in_chunk = out_chunk = ""
            html = markdown = False
    if in_chunk != "":
        yield {"in": in_chunk, "out": None, "html": False, "markdown": False}

def produce_report(code, format, color_theme, python_file_path, output_file_path):
    """
    Main function for transforming raw code
    """
    chunks = list(join_chunks(code))
    # if last chunk does not print anything, it"s appended to previous one
    if (chunks[-1]["out"] is None) and (len(chunks) > 1):
        chunks[-2]["in"] += chunks[-1]["in"]
        del chunks[-1]
    data = {
        "chunks": chunks,
        "color_theme": "light" if (format == "pdf") else color_theme,
        "file_name": python_file_path.name,
        "format": format,
        "timestamp": datetime.now().astimezone().strftime("%Y-%m-%d %H:%M:%S %Z"),
        "version": version("merkury"),
    }
    template = jinja.get_template("template.html")
    report = template.render(data)
    match format:
        case "html":
            with output_file_path.open("w") as out:
                out.write(report)
        case "pdf":
            options = {
                "page-size": "A4",
                "margin-top": "25mm",
                "margin-right": "25mm",
                "margin-bottom": "25mm",
                "margin-left": "25mm",
            }
            pdfkit.from_string(report, output_file_path, options=options)
    return True

def get_default_path(python_file_path, format):
    """
    Default file name for report
    """
    date_now = datetime.now().astimezone().strftime("%Y%m%d%H%M%S%Z")
    file_name = re.sub(".py$", "", python_file_path.name)
    out_file_name = f"{file_name}_{date_now}.{format}"
    return Path(python_file_path.parent, out_file_name)



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name="merkury",
    version="0.3",
    license="MIT",
    description="Turn Python scripts into HTML reports",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[],
    author="Piotr Patrzyk",
    url="https://github.com/ppatrzyk/merkury",
    packages=["merkury"],
    package_data={"merkury": ["templates/*", "templates/assets/*"]},
    python_requires=">=3.10",
    install_requires=[
        "docopt>=0.6.2",
        "Jinja2>=3.1.2",
        "Markdown>=3.4.1",
        "pdfkit>=1.0.0",
    ],
    entry_points={
        "console_scripts": [
            "merkury=merkury.main:main",
        ],
    },
)
